package cz.moneta.test.dsl.mwf.createaccount;

import cz.moneta.test.harness.support.mwf.Parameter;
import cz.moneta.test.harness.support.mwf.MwfRequest;

public interface MWF172 extends MwfRequest<MWF172, MWF172Response> {

    @Parameter(name = "AAAdresat")
    MWF172 aAAdresat(String aAAdresat);
    @Parameter(name = "AACO")
    MWF172 aaco(String aaco);
    @Parameter(name = "AACP")
    MWF172 aacp(String aacp);
    @Parameter(name = "AACtvrt")
    MWF172 aACtvrt(String aACtvrt);
    @Parameter(name = "AAObec")
    MWF172 aAObec(String aAObec);
    @Parameter(name = "AAPSC")
    MWF172 aapsc(String aapsc);
    @Parameter(name = "AAPosta")
    MWF172 aAPosta(String aAPosta);
    @Parameter(name = "AAUlice")
    MWF172 aAUlice(String aAUlice);
    @Parameter(name = "AAZeme")
    MWF172 aAZeme(String aAZeme);
    @Parameter(name = "BankAtWork")
    MWF172 bankAtWork(String bankAtWork);
    @Parameter(name = "CIF")
    MWF172 cif(String cif);
    @Parameter(name = "CisloUctu")
    MWF172 cisloUctu(String cisloUctu);
    @Parameter(name = "CisloUctuPrijemceKodBanky")
    MWF172 cisloUctuPrijemceKodBanky(String cisloUctuPrijemceKodBanky);
    @Parameter(name = "DatumNarozeni")
    MWF172 datumNarozeni(java.util.Date datumNarozeni);
    @Parameter(name = "DatumOtevreni")
    MWF172 datumOtevreni(java.util.Date datumOtevreni);
    @Parameter(name = "DatumPrvnihoVypisu")
    MWF172 datumPrvnihoVypisu(java.util.Date datumPrvnihoVypisu);
    @Parameter(name = "DistribuceVypisu")
    MWF172 distribuceVypisu(String distribuceVypisu);
    @Parameter(name = "DruhKlienta")
    MWF172 druhKlienta(String druhKlienta);
    @Parameter(name = "DruheJmeno")
    MWF172 druheJmeno(String druheJmeno);
    @Parameter(name = "ICO_Zamestnavatel")
    MWF172 iCOZamestnavatel(String iCOZamestnavatel);
    @Parameter(name = "Jmeno")
    MWF172 jmeno(String jmeno);
    @Parameter(name = "NazevZamestnavatele")
    MWF172 nazevZamestnavatele(String nazevZamestnavatele);
    @Parameter(name = "NuloveUroceni")
    MWF172 nuloveUroceni(String nuloveUroceni);
    @Parameter(name = "ObchodniJmeno")
    MWF172 obchodniJmeno(String obchodniJmeno);
    @Parameter(name = "ObchodniMistoAlternativni")
    MWF172 obchodniMistoAlternativni(String obchodniMistoAlternativni);
    @Parameter(name = "ObchodniMistoKod")
    MWF172 obchodniMistoKod(String obchodniMistoKod);
    @Parameter(name = "PausalniPoplatekPocetMesicuZdarma")
    MWF172 pausalniPoplatekPocetMesicuZdarma(Long pausalniPoplatekPocetMesicuZdarma);
    @Parameter(name = "PeriodaVypisu")
    MWF172 periodaVypisu(String periodaVypisu);
    @Parameter(name = "PoplatkovyPlan")
    MWF172 poplatkovyPlan(String poplatkovyPlan);
    @Parameter(name = "PoplatkovyPlanMatersky")
    MWF172 poplatkovyPlanMatersky(String poplatkovyPlanMatersky);
    @Parameter(name = "PostovniPoukazka")
    MWF172 postovniPoukazka(String postovniPoukazka);
    @Parameter(name = "PouzitaAA")
    MWF172 pouzitaAA(String pouzitaAA);
    @Parameter(name = "Prijmeni")
    MWF172 prijmeni(String prijmeni);
    @Parameter(name = "PrimarniUrednik")
    MWF172 primarniUrednik(String primarniUrednik);
    @Parameter(name = "ProduktKod")
    MWF172 produktKod(String produktKod);
    @Parameter(name = "SekundarniUrednik")
    MWF172 sekundarniUrednik(String sekundarniUrednik);
    @Parameter(name = "TitulPred")
    MWF172 titulPred(String titulPred);
    @Parameter(name = "TitulZa")
    MWF172 titulZa(String titulZa);
    @Parameter(name = "TransakceNavic")
    MWF172 transakceNavic(String transakceNavic);
    @Parameter(name = "UlozPSCBezKontroly")
    MWF172 ulozPSCBezKontroly(String ulozPSCBezKontroly);
    @Parameter(name = "UrokovaSazba")
    MWF172 urokovaSazba(Double urokovaSazba);
    @Parameter(name = "UrokovyPlan")
    MWF172 urokovyPlan(String urokovyPlan);
    @Parameter(name = "VypisNaKonciMesice")
    MWF172 vypisNaKonciMesice(Boolean vypisNaKonciMesice);
    @Parameter(name = "ZpusobPripisu")
    MWF172 zpusobPripisu(String zpusobPripisu);
}